# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'permutationTestDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_permutationTestDialog(object):
    def setupUi(self, permutationTestDialog):
        permutationTestDialog.setObjectName("permutationTestDialog")
        permutationTestDialog.resize(500, 716)
        self.gridLayout = QtWidgets.QGridLayout(permutationTestDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(permutationTestDialog)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setSizeAdjustPolicy(
            QtWidgets.QAbstractScrollArea.AdjustToContents
        )
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 466, 885))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBoxStatistics = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxStatistics.setObjectName("groupBoxStatistics")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBoxStatistics)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.labelClusterThreshold = QtWidgets.QLabel(self.groupBoxStatistics)
        self.labelClusterThreshold.setObjectName("labelClusterThreshold")
        self.gridLayout_6.addWidget(self.labelClusterThreshold, 0, 0, 1, 1)
        self.doubleSpinBoxClusterThreshold = QtWidgets.QDoubleSpinBox(
            self.groupBoxStatistics
        )
        self.doubleSpinBoxClusterThreshold.setDecimals(5)
        self.doubleSpinBoxClusterThreshold.setMaximum(1.0)
        self.doubleSpinBoxClusterThreshold.setSingleStep(0.001)
        self.doubleSpinBoxClusterThreshold.setProperty("value", 0.01)
        self.doubleSpinBoxClusterThreshold.setObjectName(
            "doubleSpinBoxClusterThreshold"
        )
        self.gridLayout_6.addWidget(self.doubleSpinBoxClusterThreshold, 0, 1, 1, 1)
        self.spinBoxNPermutations = QtWidgets.QSpinBox(self.groupBoxStatistics)
        self.spinBoxNPermutations.setMaximum(1000000000)
        self.spinBoxNPermutations.setSingleStep(100)
        self.spinBoxNPermutations.setProperty("value", 500)
        self.spinBoxNPermutations.setObjectName("spinBoxNPermutations")
        self.gridLayout_6.addWidget(self.spinBoxNPermutations, 2, 1, 1, 1)
        self.labelNPermutations = QtWidgets.QLabel(self.groupBoxStatistics)
        self.labelNPermutations.setObjectName("labelNPermutations")
        self.gridLayout_6.addWidget(self.labelNPermutations, 2, 0, 1, 1)
        self.labelClusterSignificance = QtWidgets.QLabel(self.groupBoxStatistics)
        self.labelClusterSignificance.setObjectName("labelClusterSignificance")
        self.gridLayout_6.addWidget(self.labelClusterSignificance, 1, 0, 1, 1)
        self.doubleSpinBoxClusterSignificance = QtWidgets.QDoubleSpinBox(
            self.groupBoxStatistics
        )
        self.doubleSpinBoxClusterSignificance.setDecimals(5)
        self.doubleSpinBoxClusterSignificance.setMaximum(1.0)
        self.doubleSpinBoxClusterSignificance.setSingleStep(0.001)
        self.doubleSpinBoxClusterSignificance.setProperty("value", 0.05)
        self.doubleSpinBoxClusterSignificance.setObjectName(
            "doubleSpinBoxClusterSignificance"
        )
        self.gridLayout_6.addWidget(self.doubleSpinBoxClusterSignificance, 1, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxStatistics, 3, 0, 1, 1)
        self.groupBoxChannel = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxChannel.setObjectName("groupBoxChannel")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxChannel)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.comboBoxChannelName = QtWidgets.QComboBox(self.groupBoxChannel)
        self.comboBoxChannelName.setEnabled(False)
        self.comboBoxChannelName.setObjectName("comboBoxChannelName")
        self.gridLayout_3.addWidget(self.comboBoxChannelName, 2, 1, 1, 1)
        self.radioButtonChannelType = QtWidgets.QRadioButton(self.groupBoxChannel)
        self.radioButtonChannelType.setChecked(True)
        self.radioButtonChannelType.setObjectName("radioButtonChannelType")
        self.gridLayout_3.addWidget(self.radioButtonChannelType, 0, 0, 1, 1)
        self.comboBoxChannelType = QtWidgets.QComboBox(self.groupBoxChannel)
        self.comboBoxChannelType.setObjectName("comboBoxChannelType")
        self.gridLayout_3.addWidget(self.comboBoxChannelType, 0, 1, 1, 1)
        self.radioButtonChannelName = QtWidgets.QRadioButton(self.groupBoxChannel)
        self.radioButtonChannelName.setObjectName("radioButtonChannelName")
        self.gridLayout_3.addWidget(self.radioButtonChannelName, 2, 0, 1, 1)
        self.radioButtonChannelGroup = QtWidgets.QRadioButton(self.groupBoxChannel)
        self.radioButtonChannelGroup.setObjectName("radioButtonChannelGroup")
        self.gridLayout_3.addWidget(self.radioButtonChannelGroup, 1, 0, 1, 1)
        self.comboBoxChannelGroup = QtWidgets.QComboBox(self.groupBoxChannel)
        self.comboBoxChannelGroup.setEnabled(False)
        self.comboBoxChannelGroup.setObjectName("comboBoxChannelGroup")
        self.gridLayout_3.addWidget(self.comboBoxChannelGroup, 1, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxChannel, 0, 0, 1, 1)
        self.groupBoxFrequency = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxFrequency.setObjectName("groupBoxFrequency")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxFrequency)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.doubleSpinBoxFrequencyFmin = QtWidgets.QDoubleSpinBox(
            self.groupBoxFrequency
        )
        self.doubleSpinBoxFrequencyFmin.setEnabled(False)
        self.doubleSpinBoxFrequencyFmin.setDecimals(3)
        self.doubleSpinBoxFrequencyFmin.setMinimum(-10000.0)
        self.doubleSpinBoxFrequencyFmin.setMaximum(10000.0)
        self.doubleSpinBoxFrequencyFmin.setSingleStep(0.1)
        self.doubleSpinBoxFrequencyFmin.setObjectName("doubleSpinBoxFrequencyFmin")
        self.gridLayout_5.addWidget(self.doubleSpinBoxFrequencyFmin, 0, 2, 1, 1)
        self.radioButtonFrequencyEnabled = QtWidgets.QRadioButton(
            self.groupBoxFrequency
        )
        self.radioButtonFrequencyEnabled.setObjectName("radioButtonFrequencyEnabled")
        self.gridLayout_5.addWidget(self.radioButtonFrequencyEnabled, 0, 0, 1, 1)
        self.labelFrequencyFmin = QtWidgets.QLabel(self.groupBoxFrequency)
        self.labelFrequencyFmin.setObjectName("labelFrequencyFmin")
        self.gridLayout_5.addWidget(self.labelFrequencyFmin, 0, 1, 1, 1)
        self.labelFrequencyFmax = QtWidgets.QLabel(self.groupBoxFrequency)
        self.labelFrequencyFmax.setObjectName("labelFrequencyFmax")
        self.gridLayout_5.addWidget(self.labelFrequencyFmax, 1, 1, 1, 1)
        self.radioButtonFrequencyDisabled = QtWidgets.QRadioButton(
            self.groupBoxFrequency
        )
        self.radioButtonFrequencyDisabled.setChecked(True)
        self.radioButtonFrequencyDisabled.setObjectName("radioButtonFrequencyDisabled")
        self.gridLayout_5.addWidget(self.radioButtonFrequencyDisabled, 2, 0, 1, 1)
        self.doubleSpinBoxFrequencyFmax = QtWidgets.QDoubleSpinBox(
            self.groupBoxFrequency
        )
        self.doubleSpinBoxFrequencyFmax.setEnabled(False)
        self.doubleSpinBoxFrequencyFmax.setDecimals(3)
        self.doubleSpinBoxFrequencyFmax.setMinimum(-10000.0)
        self.doubleSpinBoxFrequencyFmax.setMaximum(10000.0)
        self.doubleSpinBoxFrequencyFmax.setSingleStep(0.1)
        self.doubleSpinBoxFrequencyFmax.setObjectName("doubleSpinBoxFrequencyFmax")
        self.gridLayout_5.addWidget(self.doubleSpinBoxFrequencyFmax, 1, 2, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxFrequency, 2, 0, 1, 1)
        self.groupBoxGroups = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxGroups.setObjectName("groupBoxGroups")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxGroups)
        self.formLayout.setObjectName("formLayout")
        self.pushButtonGroups = QtWidgets.QPushButton(self.groupBoxGroups)
        self.pushButtonGroups.setObjectName("pushButtonGroups")
        self.formLayout.setWidget(
            0, QtWidgets.QFormLayout.SpanningRole, self.pushButtonGroups
        )
        self.listWidgetGroups = QtWidgets.QListWidget(self.groupBoxGroups)
        self.listWidgetGroups.setObjectName("listWidgetGroups")
        self.formLayout.setWidget(
            1, QtWidgets.QFormLayout.SpanningRole, self.listWidgetGroups
        )
        self.gridLayout_2.addWidget(self.groupBoxGroups, 4, 0, 1, 1)
        self.groupBoxTime = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxTime.setObjectName("groupBoxTime")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBoxTime)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.doubleSpinBoxTimeTmin = QtWidgets.QDoubleSpinBox(self.groupBoxTime)
        self.doubleSpinBoxTimeTmin.setEnabled(False)
        self.doubleSpinBoxTimeTmin.setDecimals(3)
        self.doubleSpinBoxTimeTmin.setMinimum(-10000.0)
        self.doubleSpinBoxTimeTmin.setMaximum(10000.0)
        self.doubleSpinBoxTimeTmin.setSingleStep(0.1)
        self.doubleSpinBoxTimeTmin.setObjectName("doubleSpinBoxTimeTmin")
        self.gridLayout_4.addWidget(self.doubleSpinBoxTimeTmin, 0, 2, 1, 1)
        self.radioButtonTimeDisabled = QtWidgets.QRadioButton(self.groupBoxTime)
        self.radioButtonTimeDisabled.setChecked(True)
        self.radioButtonTimeDisabled.setObjectName("radioButtonTimeDisabled")
        self.gridLayout_4.addWidget(self.radioButtonTimeDisabled, 2, 0, 1, 1)
        self.radioButtonTimeEnabled = QtWidgets.QRadioButton(self.groupBoxTime)
        self.radioButtonTimeEnabled.setObjectName("radioButtonTimeEnabled")
        self.gridLayout_4.addWidget(self.radioButtonTimeEnabled, 0, 0, 1, 1)
        self.labelTimeTmin = QtWidgets.QLabel(self.groupBoxTime)
        self.labelTimeTmin.setObjectName("labelTimeTmin")
        self.gridLayout_4.addWidget(self.labelTimeTmin, 0, 1, 1, 1)
        self.labelTimeTmax = QtWidgets.QLabel(self.groupBoxTime)
        self.labelTimeTmax.setObjectName("labelTimeTmax")
        self.gridLayout_4.addWidget(self.labelTimeTmax, 1, 1, 1, 1)
        self.doubleSpinBoxTimeTmax = QtWidgets.QDoubleSpinBox(self.groupBoxTime)
        self.doubleSpinBoxTimeTmax.setEnabled(False)
        self.doubleSpinBoxTimeTmax.setDecimals(3)
        self.doubleSpinBoxTimeTmax.setMinimum(-10000.0)
        self.doubleSpinBoxTimeTmax.setMaximum(10000.0)
        self.doubleSpinBoxTimeTmax.setSingleStep(0.1)
        self.doubleSpinBoxTimeTmax.setObjectName("doubleSpinBoxTimeTmax")
        self.gridLayout_4.addWidget(self.doubleSpinBoxTimeTmax, 1, 2, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxTime, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding
        )
        self.gridLayout_2.addItem(spacerItem, 6, 0, 1, 1)
        self.groupBoxDesign = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxDesign.setObjectName("groupBoxDesign")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBoxDesign)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.radioButtonWithinSubjects = QtWidgets.QRadioButton(self.groupBoxDesign)
        self.radioButtonWithinSubjects.setChecked(True)
        self.radioButtonWithinSubjects.setObjectName("radioButtonWithinSubjects")
        self.gridLayout_7.addWidget(self.radioButtonWithinSubjects, 0, 0, 1, 1)
        self.radioButtonBetweenSubjects = QtWidgets.QRadioButton(self.groupBoxDesign)
        self.radioButtonBetweenSubjects.setObjectName("radioButtonBetweenSubjects")
        self.gridLayout_7.addWidget(self.radioButtonBetweenSubjects, 1, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxDesign, 5, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(
            40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum
        )
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(permutationTestDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonApply = QtWidgets.QPushButton(permutationTestDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(permutationTestDialog)
        self.pushButtonCancel.clicked.connect(permutationTestDialog.reject)  # type: ignore
        self.pushButtonApply.clicked.connect(permutationTestDialog.accept)  # type: ignore
        self.radioButtonFrequencyDisabled.toggled["bool"].connect(self.doubleSpinBoxFrequencyFmax.setDisabled)  # type: ignore
        self.radioButtonFrequencyDisabled.toggled["bool"].connect(self.doubleSpinBoxFrequencyFmin.setDisabled)  # type: ignore
        self.radioButtonTimeDisabled.toggled["bool"].connect(self.doubleSpinBoxTimeTmin.setDisabled)  # type: ignore
        self.radioButtonTimeDisabled.toggled["bool"].connect(self.doubleSpinBoxTimeTmax.setDisabled)  # type: ignore
        self.radioButtonChannelType.toggled["bool"].connect(self.comboBoxChannelGroup.setDisabled)  # type: ignore
        self.radioButtonChannelType.toggled["bool"].connect(self.comboBoxChannelName.setDisabled)  # type: ignore
        self.radioButtonChannelGroup.toggled["bool"].connect(self.comboBoxChannelType.setDisabled)  # type: ignore
        self.radioButtonChannelGroup.toggled["bool"].connect(self.comboBoxChannelName.setDisabled)  # type: ignore
        self.radioButtonChannelName.toggled["bool"].connect(self.comboBoxChannelGroup.setDisabled)  # type: ignore
        self.radioButtonChannelName.toggled["bool"].connect(self.comboBoxChannelType.setDisabled)  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(permutationTestDialog)

    def retranslateUi(self, permutationTestDialog):
        _translate = QtCore.QCoreApplication.translate
        permutationTestDialog.setWindowTitle(
            _translate("permutationTestDialog", "Meggie - Permutation tests")
        )
        self.groupBoxStatistics.setTitle(
            _translate("permutationTestDialog", "Statistics:")
        )
        self.labelClusterThreshold.setText(
            _translate("permutationTestDialog", "Cluster threshold (p):")
        )
        self.labelNPermutations.setText(
            _translate("permutationTestDialog", "Number of permutations:")
        )
        self.labelClusterSignificance.setText(
            _translate("permutationTestDialog", "Cluster significance (p):")
        )
        self.groupBoxChannel.setTitle(
            _translate("permutationTestDialog", "Limit channels:")
        )
        self.radioButtonChannelType.setText(
            _translate("permutationTestDialog", "Channel type:")
        )
        self.radioButtonChannelName.setText(
            _translate("permutationTestDialog", "Channel:")
        )
        self.radioButtonChannelGroup.setText(
            _translate("permutationTestDialog", "Channel group:")
        )
        self.groupBoxFrequency.setTitle(
            _translate("permutationTestDialog", "Limit in frequency:")
        )
        self.radioButtonFrequencyEnabled.setText(
            _translate("permutationTestDialog", "Limit to:")
        )
        self.labelFrequencyFmin.setText(_translate("permutationTestDialog", "Fmin:"))
        self.labelFrequencyFmax.setText(_translate("permutationTestDialog", "Fmax:"))
        self.radioButtonFrequencyDisabled.setText(
            _translate("permutationTestDialog", "Do not limit")
        )
        self.groupBoxGroups.setTitle(_translate("permutationTestDialog", "Groups:"))
        self.pushButtonGroups.setText(
            _translate("permutationTestDialog", "Select groups..")
        )
        self.groupBoxTime.setTitle(
            _translate("permutationTestDialog", "Limit in time:")
        )
        self.radioButtonTimeDisabled.setText(
            _translate("permutationTestDialog", "Do not limit")
        )
        self.radioButtonTimeEnabled.setText(
            _translate("permutationTestDialog", "Limit to:")
        )
        self.labelTimeTmin.setText(_translate("permutationTestDialog", "Tmin:"))
        self.labelTimeTmax.setText(_translate("permutationTestDialog", "Tmax:"))
        self.groupBoxDesign.setTitle(_translate("permutationTestDialog", "Design:"))
        self.radioButtonWithinSubjects.setText(
            _translate("permutationTestDialog", "Within-subjects")
        )
        self.radioButtonBetweenSubjects.setText(
            _translate("permutationTestDialog", "Between-subjects")
        )
        self.pushButtonCancel.setText(_translate("permutationTestDialog", "Cancel"))
        self.pushButtonApply.setText(_translate("permutationTestDialog", "Apply"))
